@@ -110,7 +110,7 @@ TODO: Prevent crash if expr_src is unavailable when tracing...
 
 # TODO: Cache compiled ASTs!
 
-__version__ = "0.12"
+__version__ = "0.13"
 
 import sys
 import inspect
@@ -355,7 +355,8 @@ def getCurrentOutput():
 def provideInput(inputs, autoNewlines=True):
     """
     Replaces normal inputs with inputs from the given multi-line string.
-    Normal interactive input can be resumed using `restoreInput`.
+    Normal interactive input can be resumed using `restoreInput`, even if
+    multiple provideInput calls have been made.
 
     A newline will automatically be appended to the given input string if
     it doesn't already end in one, as newlines are necessary for inputs
@@ -374,17 +375,23 @@ def provideInput(inputs, autoNewlines=True):
         inputs = inputs[1:]
     if autoNewlines and not inputs.endswith('\n'):
         inputs += '\n'
-    ORIGINAL_STDIN = sys.stdin
+    # We only capture on first replacement, so that a single restoreInput
+    # call puts things back to normal.
+    if ORIGINAL_STDIN is None:
+        ORIGINAL_STDIN = sys.stdin
     sys.stdin = io.StringIO(inputs)
 
 
 def restoreInput():
     """
     Returns to normal interactive input, undoing the effects of
-    `provideInput`. Does nothing if `provideInput` has not been called.
+    `provideInput` (even if it's been called multiple times). Does
+    nothing if `provideInput` has not been called.
     """
+    global ORIGINAL_STDIN
     if ORIGINAL_STDIN:
         sys.stdin = ORIGINAL_STDIN
+        ORIGINAL_STDIN = None
 
 
 #--------------------#
